#ifndef FHGFSOPSHELPER_H_
#define FHGFSOPSHELPER_H_

#include <include_open/common/storage/Open_StorageErrors.h>
#include <include_open/common/Open_Common.h>
#include <include_closed/app/App.h>
#include <include_closed/filesystem/FsDirInfo.h>
#include <include_closed/filesystem/FsFileInfo.h>
#include <include_closed/common/toolkit/MetadataTk.h>
#include <include_closed/net/filesystem/FhgfsOpsRemoting.h>
#include <include_closed/components/InternodeSyncer.h>


extern int FhgfsOpsHelper_refreshDirInfoIncremental(App* app, const OwnerMinInfo* entryInfo,
   loff_t currentOffset, FsDirInfo* dirInfo, fhgfs_bool forceUpdate);

extern int FhgfsOpsHelper_symlink(App* app, const OwnerMinInfo* parentInfo, const char* entryName,
   const char* to, unsigned userID, unsigned groupID, int mode, OwnerMinInfo* outEntryInfo);
extern int FhgfsOpsHelper_readlink(App* app, const OwnerMinInfo* entryInfo, char __user* buf,
   int size);

extern ssize_t FhgfsOpsHelper_readStateless(App* app, const OwnerMinInfo* entryInfo,
   char __user *buf, size_t size, loff_t offset);
extern ssize_t FhgfsOpsHelper_writeStateless(App* app, const OwnerMinInfo* entryInfo,
   const char __user *buf, size_t size, loff_t offset);

extern FhgfsOpsErr FhgfsOpsHelper_flushCache(App* app, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo,
   fhgfs_bool discardCacheOnError);
extern ssize_t FhgfsOpsHelper_writeCached(App* app, const char __user *buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);
extern ssize_t FhgfsOpsHelper_readCached(App* app, char __user *buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);

extern ssize_t __FhgfsOpsHelper_writeCacheFlushed(App* app, const char __user *buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);
extern ssize_t __FhgfsOpsHelper_readCacheFlushed(App* app, char __user *buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);

extern FhgfsOpsErr FhgfsOpsHelper_readOrClearUser(App* app, char __user *buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);
extern FhgfsOpsErr FhgfsOpsHelper_readOrClearKernel(App* app, char* buf, size_t size,
   loff_t offset, FsFileInfo* fileInfo, RemotingIOInfo* ioInfo);

extern void FhgfsOpsHelper_getRelativeLinkStr(const char* pathFrom, const char* pathTo,
   char** pathRelativeTo);

// inliners
static inline FhgfsOpsErr FhgfsOpsHelper_closefileWithAsyncRetry(const OwnerMinInfo* entryInfo,
   RemotingIOInfo* ioInfo);



/**
 * Call close remoting and add close operation to the corresponding async retry queue if a
 * communucation error occured.
 *
 * @param entryInfo will be copied
 * @param ioInfo will be copied
 * @return remoting result (independent of whether the operation was added for later retry)
 */
FhgfsOpsErr FhgfsOpsHelper_closefileWithAsyncRetry(const OwnerMinInfo* entryInfo,
   RemotingIOInfo* ioInfo)
{
   FhgfsOpsErr closeRes = FhgfsOpsRemoting_closefile(entryInfo, ioInfo);

   if( (closeRes == FhgfsOpsErr_COMMUNICATION) || (closeRes == FhgfsOpsErr_INTERRUPTED) )
   { // comm error => add for later retry
      InternodeSyncer* syncer = App_getInternodeSyncer(ioInfo->app);
      InternodeSyncer_delayedCloseAdd(syncer, entryInfo, ioInfo);
   }

   return closeRes;
}


#endif /*FHGFSOPSHELPER_H_*/
